<!DOCTYPE html>
<html>
<head>
    <style>
        body {
            font-family: Arial, sans-serif;
        }
        table {
            width: 100%;
            border-collapse: collapse;
        }
        th, td {
            padding: 15px;
            text-align: left;
            border-bottom: 1px solid #ddd;
        }
        th {
            background-color: #4CAF50;
            color: white;
        }
        tr:hover {background-color: #f5f5f5;}
    </style>
    <meta charset="UTF-8">
</head>
<body>

<h1>Test Report</h1>

<p>Test Time: {{ test_time }}</p>
<p>Total Cases: {{ total_cases }}</p>
<p>Passed Cases: {{ passed_cases }}</p>

<table>
    <tr>
        <th>Case Name</th>
        <th>Case Description</th>
        <th>Case Priority</th>
        <th>Case Tag</th>
        <th>Case Status Code</th>
        <th>Case Message</th>
    </tr>
    {% for case in cases %}
    <tr>
        <td>{{ case.case_name }}</td>
        <td>{{ case.case_desc }}</td>
        <td>{{ case.priority }}</td>
        <td>{{ case.mark }}</td>
        <td>{{ case.run_status }}</td>
        <td>{{ case.run_msg }}</td>
    </tr>
    {% endfor %}
</table>

</body>
</html>