"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/23 16:56:00
"""

# config_dict = {}
# func_dict = {}
# assert_dict = {}
# action_dict = {}
# run_case_list = []
# case_load_flag = False


class BinTestData:
    """
    BinTestData class
    """
    config_dict = {}
    func_dict = {}
    assert_dict = {}
    action_dict = {}
    run_case_list = []
    case_load_flag = False
